@@ -16,7 +16,8 @@ from shenfun.utilities import inheritdocstrings
 __all__ = ['ChebyshevBase', 'Basis', 'ShenDirichletBasis',
            'ShenNeumannBasis', 'ShenBiharmonicBasis',
            'SecondNeumannBasis', 'UpperDirichletBasis',
-           'ShenBiPolarBasis', 'BCBasis', 'BCBiharmonicBasis']
+           'ShenBiPolarBasis', 'BCBasis', 'BCBiharmonicBasis',
+           'DirichletNeumannBasis']
 
 #pylint: disable=abstract-method, not-callable, method-hidden, no-self-use, cyclic-import
 
@@ -65,12 +66,21 @@ class ChebyshevBase(SpectralBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
-    def __init__(self, N, quad="GC", domain=(-1., 1.), padding_factor=1, dealias_direct=False):
+    def __init__(self, N, quad="GC", domain=(-1., 1.), padding_factor=1, dealias_direct=False,
+                 coordinates=None):
         assert quad in ('GC', 'GL')
-        self.dealias_direct = dealias_direct
-        SpectralBase.__init__(self, N, quad=quad, padding_factor=padding_factor, domain=domain)
+        SpectralBase.__init__(self, N, quad=quad, padding_factor=padding_factor,
+                              dealias_direct=dealias_direct, domain=domain, coordinates=coordinates)
 
     @staticmethod
     def family():
@@ -115,10 +125,10 @@ class ChebyshevBase(SpectralBase):
     def vandermonde(self, x):
         return n_cheb.chebvander(x, int(self.N*self.padding_factor)-1)
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         return sympy.chebyshevt(i, x)
 
-    def sympy_weight(self, x=sympy.symbols('x')):
+    def sympy_weight(self, x=sympy.symbols('x', real=True)):
         return 1/sp.sqrt(1-x**2)
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -243,7 +253,7 @@ class ChebyshevBase(SpectralBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_orthogonal(self):
-        return Basis(self.N, quad=self.quad, domain=self.domain)
+        return Basis(self.N, quad=self.quad, domain=self.domain, coordinates=self.coors.coordinates)
 
 
 @inheritdocstrings
@@ -265,12 +275,21 @@ class Basis(ChebyshevBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad='GC', domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         if quad == 'GC':
             self._xfftn_fwd = functools.partial(fftw.dctn, type=2)
             self._xfftn_bck = functools.partial(fftw.dctn, type=3)
@@ -399,6 +418,14 @@ class ShenDirichletBasis(ChebyshevBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
 
     Note
     ----
@@ -407,9 +434,10 @@ class ShenDirichletBasis(ChebyshevBase):
     """
 
     def __init__(self, N, quad="GC", bc=(0, 0), domain=(-1., 1.), scaled=False,
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         from shenfun.tensorproductspace import BoundaryValues
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self._scaled = scaled
@@ -429,7 +457,8 @@ class ShenDirichletBasis(ChebyshevBase):
         return self.__class__(N, quad=self.quad,
                               domain=self.domain, padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
-                              bc=self.bc.bc, scaled=self._scaled)
+                              bc=self.bc.bc, scaled=self._scaled,
+                              coordinates=self.coors.coordinates)
 
     def _composite_basis(self, V, argument=0):
         P = np.zeros_like(V)
@@ -439,7 +468,7 @@ class ShenDirichletBasis(ChebyshevBase):
             P[:, -2] = (V[:, 0] - V[:, 1])/2    # x = -1
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         assert i < self.N
         if i < self.N-2:
             return sympy.chebyshevt(i, x) - sympy.chebyshevt(i+2, x)
@@ -511,6 +540,8 @@ class ShenDirichletBasis(ChebyshevBase):
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
             output_array = np.zeros_like(input_array.__array__())
+        else:
+            output_array.fill(0)
         s0 = self.sl[slice(0, -2)]
         s1 = self.sl[slice(2, None)]
         output_array[s0] = input_array[s0]
@@ -589,7 +620,7 @@ class ShenDirichletBasis(ChebyshevBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_bc_basis(self):
-        return BCBasis(self.N, quad=self.quad, domain=self.domain)
+        return BCBasis(self.N, quad=self.quad, domain=self.domain, coordinates=self.coors.coordinates)
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         return ShenDirichletBasis(self.N,
@@ -597,7 +628,8 @@ class ShenDirichletBasis(ChebyshevBase):
                                   padding_factor=padding_factor,
                                   dealias_direct=dealias_direct,
                                   domain=self.domain,
-                                  bc=self.bc.bc)
+                                  bc=self.bc.bc,
+                                  coordinates=self.coors.coordinates)
 
     def _truncation_forward(self, padded_array, trunc_array):
         if not id(trunc_array) == id(padded_array):
@@ -644,11 +676,20 @@ class ShenNeumannBasis(ChebyshevBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="GC", mean=0, domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         self.mean = mean
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self._factor = np.zeros(0)
@@ -662,6 +703,7 @@ class ShenNeumannBasis(ChebyshevBase):
         return self.__class__(N, quad=self.quad,
                               domain=self.domain, padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates,
                               mean=self.mean)
 
     def _composite_basis(self, V, argument=0):
@@ -671,7 +713,7 @@ class ShenNeumannBasis(ChebyshevBase):
         P[:, :-2] = V[:, :-2] - (k[:-2]/(k[:-2]+2))**2*V[:, 2:]
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if 0 < i < self.N-2:
             return sympy.chebyshevt(i, x) - (i/(i+2))**2*sympy.chebyshevt(i+2, x)
         return 0
@@ -761,6 +803,8 @@ class ShenNeumannBasis(ChebyshevBase):
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
             output_array = np.zeros_like(input_array.v)
+        else:
+            output_array.fill(0)
         s0 = self.sl[slice(0, -2)]
         s1 = self.sl[slice(2, None)]
         self.set_factor_array(input_array)
@@ -842,10 +886,11 @@ class ShenBiharmonicBasis(ChebyshevBase):
 
     """
     def __init__(self, N, quad="GC", bc=(0, 0, 0, 0), domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         from shenfun.tensorproductspace import BoundaryValues
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self._factor1 = np.zeros(0)
         self._factor2 = np.zeros(0)
@@ -868,7 +913,7 @@ class ShenBiharmonicBasis(ChebyshevBase):
             P[:, -4:] = np.tensordot(V[:, :4], BCBiharmonicBasis.coefficient_matrix(), (1, 1))
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if i < self.N-4:
             f = sympy.chebyshevt(i, x) - (2*(i+2)/(i+3))*sympy.chebyshevt(i+2, x) + (i+1)/(i+3)*sympy.chebyshevt(i+4, x)
         else:
@@ -961,6 +1006,8 @@ class ShenBiharmonicBasis(ChebyshevBase):
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
             output_array = np.zeros_like(input_array.v)
+        else:
+            output_array.fill(0)
         self.set_factor_arrays(input_array)
         output_array = self.set_w_hat(output_array, input_array, self._factor1, self._factor2)
         self.bc.add_to_orthogonal(output_array, input_array)
@@ -1010,7 +1057,7 @@ class ShenBiharmonicBasis(ChebyshevBase):
         return output_array
 
     def get_bc_basis(self):
-        return BCBiharmonicBasis(self.N, quad=self.quad, domain=self.domain)
+        return BCBiharmonicBasis(self.N, quad=self.quad, domain=self.domain, coordinates=self.coors.coordinates)
 
     def plan(self, shape, axis, dtype, options):
         if isinstance(axis, tuple):
@@ -1087,12 +1134,21 @@ class SecondNeumannBasis(ChebyshevBase): #pragma: no cover
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="GC", mean=0, domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         self.mean = mean
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self._factor = np.zeros(0)
@@ -1106,6 +1162,7 @@ class SecondNeumannBasis(ChebyshevBase): #pragma: no cover
         return self.__class__(N, quad=self.quad,
                               domain=self.domain, padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates,
                               mean=self.mean)
 
     def _composite_basis(self, V, argument=0):
@@ -1115,7 +1172,7 @@ class SecondNeumannBasis(ChebyshevBase): #pragma: no cover
         P[:, :-2] = V[:, :-2] - (k[:-2]/(k[:-2]+2))**2*(k[:-2]**2-1)/((k[:-2]+2)**2-1)*V[:, 2:]
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         return sympy.chebyshevt(i, x) - (i/(i+2))**2*(i**2-1)/((i+2)**2-1)*sympy.chebyshevt(i+2, x)
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -1189,6 +1246,8 @@ class SecondNeumannBasis(ChebyshevBase): #pragma: no cover
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
             output_array = np.zeros_like(input_array.v)
+        else:
+            output_array.fill(0)
         s0 = self.sl[slice(0, -2)]
         s1 = self.sl[slice(2, None)]
         self.set_factor_array(input_array)
@@ -1264,13 +1323,22 @@ class UpperDirichletBasis(ChebyshevBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
 
     """
 
     def __init__(self, N, quad="GC", domain=(-1., 1.), scaled=False,
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self._scaled = scaled
         self._factor = np.ones(1)
@@ -1288,6 +1356,7 @@ class UpperDirichletBasis(ChebyshevBase):
         return self.__class__(N, quad=self.quad,
                               domain=self.domain, padding_factor=self.padding_factor,
                               dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates,
                               scaled=self._scaled)
 
     def _composite_basis(self, V, argument=0):
@@ -1295,7 +1364,7 @@ class UpperDirichletBasis(ChebyshevBase):
         P[:, :-1] = V[:, :-1] - V[:, 1:]
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         assert i < self.N-1
         return sympy.chebyshevt(i, x) - sympy.chebyshevt(i+1, x)
 
@@ -1336,6 +1405,7 @@ class UpperDirichletBasis(ChebyshevBase):
         s0 = self.sl[slice(0, -1)]
         s1 = self.sl[slice(1, None)]
         output[s0] -= output[s1]
+        output[self.si[-1]] = 0
 
     def evaluate_expansion_all(self, input_array, output_array, fast_transform=True):
         if fast_transform is False:
@@ -1353,6 +1423,8 @@ class UpperDirichletBasis(ChebyshevBase):
     def to_ortho(self, input_array, output_array=None):
         if output_array is None:
             output_array = np.zeros_like(input_array.__array__())
+        else:
+            output_array.fill(0)
         s0 = self.sl[slice(0, -1)]
         s1 = self.sl[slice(1, None)]
         output_array[s0] = input_array[s0]
@@ -1427,11 +1499,12 @@ class UpperDirichletBasis(ChebyshevBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
-        return PolarBasis(self.N,
-                          quad=self.quad,
-                          padding_factor=padding_factor,
-                          dealias_direct=dealias_direct,
-                          domain=self.domain)
+        return UpperDirichletBasis(self.N,
+                                   quad=self.quad,
+                                   padding_factor=padding_factor,
+                                   dealias_direct=dealias_direct,
+                                   coordinates=self.coors.coordinates,
+                                   domain=self.domain)
 
 
 @inheritdocstrings
@@ -1454,12 +1527,21 @@ class ShenBiPolarBasis(ChebyshevBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="GC", domain=(-1., 1.),
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         #assert quad == "GC"
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               padding_factor=padding_factor, dealias_direct=dealias_direct,
+                               coordinates=coordinates)
         self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
         self.plan(int(N*padding_factor), 0, np.float, {})
 
@@ -1477,7 +1559,7 @@ class ShenBiPolarBasis(ChebyshevBase):
     def slice(self):
         return slice(0, self.N-4)
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         f = (1-x)**2*(1+x)**2*(sympy.chebyshevt(i+1, x).diff(x, 1))
         return f
 
@@ -1485,7 +1567,7 @@ class ShenBiPolarBasis(ChebyshevBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, X)
         output_array[:] = sympy.lambdify(X, f)(x)
         return output_array
@@ -1493,10 +1575,9 @@ class ShenBiPolarBasis(ChebyshevBase):
     def evaluate_basis_all(self, x=None, argument=0):
         if x is None:
             x = self.mesh(False, False)
-        N, M = self.shape(False), self.shape(True)
-        output_array = np.zeros((M, N))
-        D = np.zeros(M)
-        for j in range(N-4):
+        output_array = np.zeros((x.shape[0], self.N))
+        D = np.zeros(x.shape[0])
+        for j in range(self.N-4):
             output_array[:, j] = self.evaluate_basis(x, j, D)
         return output_array
 
@@ -1506,7 +1587,7 @@ class ShenBiPolarBasis(ChebyshevBase):
         if output_array is None:
             output_array = np.zeros(x.shape)
         x = np.atleast_1d(x)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, X).diff(X, k)
         output_array[:] = sympy.lambdify(X, f)(x)
         return output_array
@@ -1514,11 +1595,10 @@ class ShenBiPolarBasis(ChebyshevBase):
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
             x = self.mesh(False, False)
-        N, M = self.shape(False), self.shape(True)
-        output_array = np.zeros((M, N))
-        D = np.zeros(M)
-        for j in range(N-4):
-            output_array[:, j] = self.evaluate_basis_derivative(x, j, k, D)
+        output_array = np.zeros((x.shape[0], self.N))
+        D = np.zeros(x.shape[0])
+        for j in range(self.N-4):
+            output_array[:, j] = self.evaluate_basis_derivative(x, j, k, output_array=D)
         return output_array
 
     def scalar_product(self, input_array=None, output_array=None, fast_transform=False):
@@ -1534,8 +1614,8 @@ class ShenBiPolarBasis(ChebyshevBase):
         if output_array is None:
             output_array = np.zeros(x.shape)
         x = self.map_reference_domain(x)
-        fj = self.eval_basis_all(x)
-        output_array[:] = np.dot(u, fj)
+        fj = self.evaluate_basis_all(x)
+        output_array[:] = np.dot(fj, u)
         return output_array
 
     def forward(self, input_array=None, output_array=None, fast_transform=False):
@@ -1583,8 +1663,239 @@ class ShenBiPolarBasis(ChebyshevBase):
                                 quad=self.quad,
                                 padding_factor=padding_factor,
                                 dealias_direct=dealias_direct,
+                                coordinates=self.coors.coordinates,
                                 domain=self.domain)
 
+@inheritdocstrings
+class DirichletNeumannBasis(ChebyshevBase):
+    """Basis for mixed Dirichlet/Neumann boundary conditions
+
+    Parameters
+    ----------
+        N : int, optional
+            Number of quadrature points
+        quad : str, optional
+            Type of quadrature
+
+            - GL - Chebyshev-Gauss-Lobatto
+            - GC - Chebyshev-Gauss
+
+        domain : 2-tuple of floats, optional
+            The computational domain
+        scaled : bool, optional
+            Whether or not to use scaled basis
+        padding_factor : float, optional
+            Factor for padding backward transforms.
+        dealias_direct : bool, optional
+            Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
+
+    """
+
+    def __init__(self, N, quad="GC", domain=(-1., 1.), scaled=False,
+                 padding_factor=1, dealias_direct=False, coordinates=None):
+        ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
+                               padding_factor=padding_factor, dealias_direct=dealias_direct, coordinates=coordinates)
+        self.CT = Basis(N, quad=quad, padding_factor=padding_factor, dealias_direct=dealias_direct)
+        self._scaled = scaled
+        self._factor1 = np.zeros(0)
+        self._factor2 = np.zeros(0)
+        self.plan(int(N*padding_factor), 0, np.float, {})
+
+    @staticmethod
+    def boundary_condition():
+        return 'DirichletNeumann'
+
+    @property
+    def has_nonhomogeneous_bcs(self):
+        return False
+
+    def get_refined(self, N):
+        return self.__class__(N, quad=self.quad,
+                              domain=self.domain, padding_factor=self.padding_factor,
+                              dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates,
+                              scaled=self._scaled)
+
+    def set_factor_arrays(self, v):
+        """Set intermediate factor arrays"""
+        s = self.sl[self.slice()]
+        if not self._factor1.shape == v[s].shape:
+            k = self.wavenumbers().astype(float)
+            self._factor1 = ((-k**2 + (k + 2)**2)/((k + 1)**2 + (k + 2)**2)).astype(float)
+            self._factor2 = ((-k**2 - (k + 1)**2)/((k + 1)**2 + (k + 2)**2)).astype(float)
+
+    def _composite_basis(self, V, argument=0):
+        P = np.zeros_like(V)
+        k = np.arange(V.shape[1]).astype(np.float)[:-2]
+        P[:, :-2] = (V[:, :-2] +
+                     ((-k**2 + (k + 2)**2)/((k + 1)**2 + (k + 2)**2))*V[:, 1:-1] +
+                     ((-k**2 - (k + 1)**2)/((k + 1)**2 + (k + 2)**2))*V[:, 2:])
+        return P
+
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
+        assert i < self.N-2
+        return (sympy.chebyshevt(i, x)+
+               ((-i**2+(i+2)**2)/((i+1)**2+(i+2)**2))*sympy.chebyshevt(i+1, x)+
+               ((-i**2-(i+1)**2)/((i+1)**2+(i+2)**2))*sympy.chebyshevt(i+2, x))
+
+    def evaluate_basis(self, x, i=0, output_array=None):
+        x = np.atleast_1d(x)
+        if output_array is None:
+            output_array = np.zeros(x.shape)
+        w = np.arccos(x)
+        output_array[:] = (np.cos(i*w)+
+                           ((-i**2+(i+2)**2)/((i+1)**2+(i+2)**2))*np.cos((i+1)*w)+
+                           ((-i**2-(i+1)**2)/((i+1)**2+(i+2)**2))*np.cos((i+2)*w))
+        return output_array
+
+    def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
+        if x is None:
+            x = self.mesh(False, False)
+        if output_array is None:
+            output_array = np.zeros(x.shape)
+        x = np.atleast_1d(x)
+        basis = np.zeros(self.shape(True))
+        basis[np.array([i, i+1, i+2])] = (1,((-i**2+(i+2)**2)/((i+1)**2+(i+2)**2)), ((-i**2-(i+1)**2)/((i+1)**2+(i+2)**2)))
+        basis = n_cheb.Chebyshev(basis)
+        if k > 0:
+            basis = basis.deriv(k)
+        output_array[:] = basis(x)
+        return output_array
+
+    def is_scaled(self):
+        """Return True if scaled basis is used, otherwise False"""
+        return False
+
+    def set_w_hat(self, w_hat, fk, f1, f2):
+        """Return intermediate w_hat array"""
+        s0 = self.sl[slice(0, -2)]
+        s1 = self.sl[slice(1, -1)]
+        s2 = self.sl[slice(2, None)]
+        w_hat[s0] = fk[s0]
+        w_hat[s1] += f1*fk[s0]
+        w_hat[s2] += f2*fk[s0]
+        return w_hat
+
+    def evaluate_scalar_product(self, input_array, output_array, fast_transform=True):
+        if fast_transform is False:
+            self.vandermonde_scalar_product(input_array, output_array)
+            return
+        output = self.CT.scalar_product(fast_transform=fast_transform)
+        Tk = work[(output, 0, True)]
+        Tk[...] = output
+        self.set_factor_arrays(Tk)
+        s0 = self.sl[slice(0, -2)]
+        s1 = self.sl[slice(1, -1)]
+        s2 = self.sl[slice(2, None)]
+        output[s0] += self._factor1*Tk[s1]
+        output[s0] += self._factor2*Tk[s2]
+
+    def evaluate_expansion_all(self, input_array, output_array, fast_transform=True):
+        if fast_transform is False:
+            self._padding_backward(input_array, self.backward.tmp_array)
+            SpectralBase.evaluate_expansion_all(self, self.backward.tmp_array, output_array, False)
+            return
+        w_hat = work[(input_array, 0, True)]
+        self.set_factor_arrays(input_array)
+        w_hat = self.set_w_hat(w_hat, input_array, self._factor1, self._factor2)
+        self.CT.backward(w_hat)
+
+    def to_ortho(self, input_array, output_array=None):
+        if output_array is None:
+            output_array = np.zeros_like(input_array.__array__())
+        else:
+            output_array.fill(0)
+        self.set_factor_arrays(input_array)
+        output_array = self.set_w_hat(output_array, input_array, self._factor1, self._factor2)
+        return output_array
+
+    def slice(self):
+        return slice(0, self.N-2)
+
+    def eval(self, x, u, output_array=None):
+        x = np.atleast_1d(x)
+        if output_array is None:
+            output_array = np.zeros(x.shape)
+        x = self.map_reference_domain(x)
+        w_hat = work[(u, 0, True)]
+        self.set_factor_arrays(w_hat)
+        output_array[:] = n_cheb.chebval(x, u[:-2])
+        w_hat[1:-1] = self._factor1*u[:-2]
+        output_array += n_cheb.chebval(x, w_hat)
+        w_hat[2:] = self._factor2*u[:-2]
+        w_hat[:2] = 0
+        output_array += n_cheb.chebval(x, w_hat)
+        return output_array
+
+    def forward(self, input_array=None, output_array=None, fast_transform=True):
+        self.scalar_product(input_array, fast_transform=fast_transform)
+        u = self.scalar_product.tmp_array
+        self.apply_inverse_mass(u)
+        self._truncation_forward(u, self.forward.output_array)
+        if output_array is not None:
+            output_array[...] = self.forward.output_array
+            return output_array
+        return self.forward.output_array
+
+    def backward(self, input_array=None, output_array=None, fast_transform=True):
+        if input_array is not None:
+            self.backward.input_array[...] = input_array
+
+        self.evaluate_expansion_all(self.backward.input_array,
+                                    self.backward.output_array,
+                                    fast_transform=fast_transform)
+
+        if output_array is not None:
+            output_array[...] = self.backward.output_array
+            return output_array
+        return self.backward.output_array
+
+    def plan(self, shape, axis, dtype, options):
+        if isinstance(axis, tuple):
+            assert len(axis) == 1
+            axis = axis[-1]
+
+        if isinstance(self.forward, Transform):
+            if self.forward.input_array.shape == shape and self.axis == axis:
+                # Already planned
+                return
+
+        self.CT.plan(shape, axis, dtype, options)
+        self.CT.tensorproductspace = self.tensorproductspace
+        xfftn_fwd = self.CT.forward.xfftn
+        xfftn_bck = self.CT.backward.xfftn
+        U = xfftn_fwd.input_array
+        V = xfftn_fwd.output_array
+        self.axis = axis
+        if self.padding_factor > 1.+1e-8:
+            trunc_array = self._get_truncarray(shape, V.dtype)
+            self.forward = Transform(self.forward, xfftn_fwd, U, V, trunc_array)
+            self.backward = Transform(self.backward, xfftn_bck, trunc_array, V, U)
+            self.backward_uniform = Transform(self.backward_uniform, xfftn_bck, trunc_array, V, U)
+        else:
+            self.forward = Transform(self.forward, xfftn_fwd, U, V, V)
+            self.backward = Transform(self.backward, xfftn_bck, V, V, U)
+            self.backward_uniform = Transform(self.backward_uniform, xfftn_bck, V, V, U)
+        self.scalar_product = Transform(self.scalar_product, xfftn_fwd, U, V, V)
+        self.si = islicedict(axis=self.axis, dimensions=self.dimensions)
+        self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
+
+    def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
+        return DirichletNeumannBasis(self.N,
+                                     quad=self.quad,
+                                     padding_factor=padding_factor,
+                                     dealias_direct=dealias_direct,
+                                     coordinates=self.coors.coordinates,
+                                     domain=self.domain)
+
 
 @inheritdocstrings
 class BCBasis(ChebyshevBase):
@@ -1604,16 +1915,20 @@ class BCBasis(ChebyshevBase):
             The computational domain
         scaled : bool, optional
             Whether or not to use scaled basis
-        padding_factor : float, optional
-            Factor for padding backward transforms.
-        dealias_direct : bool, optional
-            Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="GC", domain=(-1., 1.), scaled=False,
-                 padding_factor=1, dealias_direct=False):
+                 coordinates=None):
         ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                               coordinates=coordinates)
         self.plan(N, 0, np.float, {})
 
     def plan(self, shape, axis, dtype, options):
@@ -1660,7 +1975,7 @@ class BCBasis(ChebyshevBase):
         P[:] = np.tensordot(V[:, :2], self.coefficient_matrix(), (1, 1))
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if i == 0:
             return 0.5*(1-x)
         elif i == 1:
@@ -1708,16 +2023,19 @@ class BCBiharmonicBasis(ChebyshevBase):
             The computational domain
         scaled : bool, optional
             Whether or not to use scaled basis
-        padding_factor : float, optional
-            Factor for padding backward transforms.
-        dealias_direct : bool, optional
-            Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="GC", domain=(-1., 1.), scaled=False,
-                 padding_factor=1, dealias_direct=False):
-        ChebyshevBase.__init__(self, N, quad=quad, domain=domain,
-                               padding_factor=padding_factor, dealias_direct=dealias_direct)
+                 coordinates=None):
+        ChebyshevBase.__init__(self, N, quad=quad, domain=domain, coordinates=coordinates)
         self.plan(N, 0, np.float, {})
 
     def plan(self, shape, axis, dtype, options):
@@ -1767,7 +2085,7 @@ class BCBiharmonicBasis(ChebyshevBase):
         P = np.tensordot(V[:, :4], self.coefficient_matrix(), (1, 1))
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         assert i < 4, 'Only four bases, i < 4'
         return self.coefficient_matrix()[i]*np.array([sympy.chebyshevt(j, x) for j in range(4)])
 
@@ -1775,7 +2093,7 @@ class BCBiharmonicBasis(ChebyshevBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, x=X)
         output_array[:] = sympy.lambdify(X, f)(x)
         return output_array
@@ -1784,7 +2102,7 @@ class BCBiharmonicBasis(ChebyshevBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, x=X)
         output_array[:] = sympy.lambdify(X, f.diff(X, k))(x)
         return output_array
