@@ -13,25 +13,18 @@ class TDMA(la_TDMA):
 
     def __call__(self, b, u=None, axis=0):
 
-        v = self.mat.testfunction[0]
-        bc = v.bc
-
         if u is None:
             u = b
         else:
             assert u.shape == b.shape
             u[:] = b[:]
 
-        bc.apply_before(u, False, scales=(-np.pi/2., -np.pi/4.))
-
         if not self.dd.shape[0] == self.mat.shape[0]:
             self.init()
 
         self.TDMA_SymSolve(self.dd, self.ud, self.L, u, axis=axis)
 
-        bc.apply_after(u, False)
-
-        if not self.mat.scale in (1, 1.0):
+        if self.mat.scale not in (1, 1.0):
             u /= self.mat.scale
         return u
 
@@ -140,6 +133,16 @@ class Helmholtz(object):
     """
     def __init__(self, *args):
 
+        args = list(args)
+        for i, arg in enumerate(args):
+            if hasattr(arg, 'is_bc_matrix'):
+                if arg.is_bc_matrix():
+                    # For this particular case the boundary dofs contribution
+                    # to the right hand side is only nonzero for Fourier wavenumber
+                    # 0, so the contribution is in effect zero
+                    args.pop(i)
+                    break
+
         assert len(args) in (2, 4)
         A, B = args[0], args[1]
         M = {d.get_key(): d for d in (A, B)}
@@ -152,6 +155,8 @@ class Helmholtz(object):
             if isinstance(self.A, TPMatrix):
                 self.A = self.A.pmat
                 self.B = self.B.pmat
+                self.alfa *= self.A.scale
+                self.beta *= self.B.scale
         elif len(args) == 4:
             self.alfa = args[2]
             self.beta = args[3]
@@ -161,13 +166,11 @@ class Helmholtz(object):
         B[-2] = np.broadcast_to(B[-2], A[2].shape)
         v = A.testfunction[0]
         neumann = self.neumann = v.boundary_condition() == 'Neumann'
-        if not self.neumann:
-            self.bc = v.bc
         self.axis = A.axis
         shape = [1]
         T = A.tensorproductspace
         if T is not None:
-            shape = list(T.local_shape(True))
+            shape = list(T.shape(True))
             shape[A.axis] = 1
         self.alfa = np.atleast_1d(self.alfa)
         self.beta = np.atleast_1d(self.beta)
@@ -209,11 +212,10 @@ class Helmholtz(object):
         determined on creation of the class.
 
         """
-
         self.Solve_Helmholtz(b, u, self.neumann, self.u0, self.u1, self.u2, self.L, self.axis)
 
-        if not self.neumann:
-            self.bc.apply_after(u, True)
+        if self.A.testfunction[0].has_nonhomogeneous_bcs:
+            self.A.testfunction[0].bc.set_boundary_dofs(u, True)
 
         return u
 
@@ -329,7 +331,13 @@ class Biharmonic(object):
     """
     def __init__(self, *args):
 
+        args = list(args)
+
+        if isinstance(args[0], TPMatrix):
+            args = [arg for arg in args if not arg.is_bc_matrix()]
+
         assert len(args) in (3, 6)
+
         S, A, B = args[0], args[1], args[2]
         M = {d.get_key(): d for d in (S, A, B)}
         self.S = M['SBBmat']
@@ -344,6 +352,8 @@ class Biharmonic(object):
                 self.S = self.S.pmat
                 self.A = self.A.pmat
                 self.B = self.B.pmat
+                self.alfa *= self.A.scale
+                self.beta *= self.B.scale
         elif len(args) == 6:
             self.a0 = a0 = args[3]
             self.alfa = alfa = args[4]
@@ -355,7 +365,7 @@ class Biharmonic(object):
         if T is None:
             shape = [S[0].shape]
         else:
-            shape = list(T.local_shape(True))
+            shape = list(T.shape(True))
         sii, siu, siuu = S[0], S[2], S[4]
         ail, aii, aiu = A[-2], A[0], A[2]
         bill, bil, bii, biu, biuu = B[-4], B[-2], B[0], B[2], B[4]
@@ -416,6 +426,8 @@ class Biharmonic(object):
         """
         self.Biharmonic_Solve(b, u, self.u0, self.u1, self.u2, self.l0,
                               self.l1, self.ak, self.bk, self.a0, self.axis)
+        if self.S.testfunction[0].has_nonhomogeneous_bcs:
+            self.S.testfunction[0].bc.set_boundary_dofs(u, True)
 
         return u
 
