@@ -65,8 +65,7 @@ To see that this is in fact the BDDmat:
 
 from __future__ import division
 
-#__all__ = ['mat']
-
+import functools
 import numpy as np
 import sympy as sp
 from shenfun.optimization import cython
@@ -363,7 +362,7 @@ class BTTmat(SpectralMatrix):
             format = 'csr'
         if format == 'self':
             s = [np.newaxis,]*v.ndim # broadcasting
-            d = [slice(0, m) for m in v.shape]
+            d = tuple(slice(0, m) for m in v.shape)
             N, M = self.shape
             s[axis] = slice(0, M)
             s = tuple(s)
@@ -1300,7 +1299,8 @@ class _ChebMatDict(dict):
     """
 
     def __missing__(self, key):
-        c = _Chebmatrix
+        measure = 1 if len(key) == 2 else key[3]
+        c = functools.partial(_Chebmatrix, measure=measure)
         self[key] = c
         return c
 
