@@ -1,4 +1,5 @@
 from numbers import Number, Integral
+from scipy.special import sph_harm
 import numpy as np
 import sympy as sp
 from shenfun.optimization.cython import evaluate
@@ -7,8 +8,13 @@ from mpi4py_fft import DistArray
 __all__ = ('Expr', 'BasisFunction', 'TestFunction', 'TrialFunction', 'Function',
            'Array', 'Basis')
 
+# Define some special functions required for spherical harmonics
+cot = lambda x: 1/np.tan(x)
+Ynm = lambda n, m, x, y : sph_harm(m, n, y, x)
+
 def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
-          scaled=None, padding_factor=1.0, dealias_direct=False, **kw):
+          scaled=None, padding_factor=1.0, dealias_direct=False,
+          coordinates=None, **kw):
     """Return basis for one dimension
 
     Parameters
@@ -63,6 +69,16 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
         only for Fourier)
     dealias_direct : bool, optional
         Use 2/3-rule dealiasing (only Fourier)
+    cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
+
+        where theta and rv are the first and second items in the 2-tuple.
 
     Examples
     --------
@@ -71,14 +87,14 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
     >>> C1 = Basis(32, 'C', quad='GC')
 
     """
-    par = {}
+    par = {'padding_factor': padding_factor,
+           'dealias_direct': dealias_direct,
+           'coordinates': coordinates}
     par.update(kw)
     if domain is not None:
         par['domain'] = domain
     if family.lower() in ('fourier', 'f'):
         from shenfun import fourier
-        par.update({'padding_factor': padding_factor,
-                    'dealias_direct': dealias_direct})
         if np.dtype(dtype).char in 'FDG':
             B = fourier.bases.C2CBasis
         else:
@@ -115,6 +131,8 @@ def Basis(N, family='Fourier', bc=None, dtype='d', quad=None, domain=None,
                 B = chebyshev.bases.UpperDirichletBasis
             elif bc.lower() == 'bipolar':
                 B = chebyshev.bases.ShenBiPolarBasis
+            elif bc.lower() == 'dirichletneumann':
+                B = chebyshev.bases.DirichletNeumannBasis
 
         else:
             raise NotImplementedError
@@ -250,7 +268,7 @@ class Expr(object):
     terms : Numpy array of ndim = 3
         Describes operations performed in Expr
 
-        - Index 0: Vector component. If Expr is rank = 0, then terms[0] = 1.
+        - Index 0: Vector component. If Expr is rank = 0, then terms.shape[0] = 1.
           For vectors it equals ndim
 
         - Index 1: One for each term in the form. For example `div(grad(u))`
@@ -276,7 +294,8 @@ class Expr(object):
         second direction, etc.
 
     scales :  Numpy array of shape == terms.shape[:2]
-        Representing a scalar multiply of each inner product
+        Representing a scalar multiply of each inner product. Note that
+        the scalar can be a function of coordinates (using sympy).
 
     indices : Numpy array of shape == terms.shape[:2]
         Index into MixedTensorProductSpace. Only used when basis of form has
@@ -310,22 +329,18 @@ class Expr(object):
     final 3 since it is a 3-dimensional tensor product space.
     """
 
-    def __init__(self, basis, terms=None, scales=None, indices=None, measures=None):
+    def __init__(self, basis, terms=None, scales=None, indices=None):
         #assert isinstance(basis, BasisFunction)
         self._basis = basis
         self._terms = terms
         self._scales = scales
         self._indices = indices
-        self._measures = measures
         ndim = self.function_space().dimensions
         if terms is None:
             self._terms = np.zeros((self.function_space().num_components(), 1, ndim),
                                    dtype=np.int)
         if scales is None:
-            self._scales = np.ones((self.function_space().num_components(), 1))
-
-        if measures is None:
-            self._measures = np.ones((self.function_space().num_components(), 1), dtype=object)
+            self._scales = np.ones((self.function_space().num_components(), 1), dtype=object)
 
         if indices is None:
             self._indices = basis.offset()+np.arange(self.function_space().num_components())[:, np.newaxis]
@@ -429,7 +444,6 @@ class Expr(object):
         for vec, (base, ind) in enumerate(zip(self.terms(), self.indices())):
             for base_j, b0 in enumerate(base):
                 M = []
-                sc = self.scales()[vec, base_j]
                 test_sp = V
                 if isinstance(V, MixedTensorProductSpace):
                     test_sp = V.flatten()[ind[base_j]]
@@ -439,6 +453,8 @@ class Expr(object):
                 for axis, k in enumerate(b0):
                     xx = test_sp[axis].map_reference_domain(np.squeeze(x[axis]))
                     P = test_sp[axis].evaluate_basis_derivative_all(xx, k=k)
+                    if not test_sp[axis].domain_factor() == 1:
+                        P *= test_sp[axis].domain_factor()**(k)
                     M.append(P[..., V.local_slice()[axis]])
                     if isinstance(test_sp[axis], R2CBasis):
                         r2c = axis
@@ -456,6 +472,17 @@ class Expr(object):
 
                 elif len(x) == 3:
                     work = evaluate.evaluate_3D(work, bv, M, r2c, last_conj_index, sl)
+
+                sc = self.scales()[vec, base_j]
+                if not hasattr(sc, 'free_symbols'):
+                    sc = float(sc)
+                else:
+                    sym0 = sc.free_symbols
+                    m = []
+                    for sym in sym0:
+                        j = 'xyzrs'.index(str(sym))
+                        m.append(x[j])
+                    sc = sp.lambdify(sym0, sc)(*m)
                 output_array += sc*work
 
         return output_array
@@ -471,97 +498,49 @@ class Expr(object):
             return Expr(basis,
                         self._terms[i][np.newaxis, :, :],
                         self._scales[i][np.newaxis, :],
-                        self._indices[i][np.newaxis, :],
-                        self._measures[i][np.newaxis, :])
+                        self._indices[i][np.newaxis, :])
+
         elif self.expr_rank() == 2:
             ndim = self.dimensions
             return Expr(basis,
                         self._terms[i*ndim:(i+1)*ndim],
                         self._scales[i*ndim:(i+1)*ndim],
-                        self._indices[i*ndim:(i+1)*ndim],
-                        self._measures[i*ndim:(i+1)*ndim])
+                        self._indices[i*ndim:(i+1)*ndim])
         else:
             raise NotImplementedError
 
     def __mul__(self, a):
         sc = self.scales().copy()
-        ms = self.measures().copy()
         if self.expr_rank() == 0:
-            if isinstance(a, sp.Expr):
-                if isinstance(a, Number):
-                    sc = sc*float(a)
-                else:
-                    ms = ms*a
-            elif isinstance(a, Number):
-                sc = sc*a
-            else:
-                raise NotImplementedError
+            sc = sc * sp.sympify(a)
+
         else:
             if isinstance(a, tuple):
                 assert len(a) == self.num_components()
                 for i in range(self.num_components()):
-                    if isinstance(a[i], sp.Expr):
-                        if isinstance(a[i], Number):
-                            sc[i] = sc[i]*float(a[i])
-                        else:
-                            ms[i] = ms[i]*a[i]
-                    elif isinstance(a[i], Number):
-                        sc[i] = sc[i]*a[i]
-
-            elif isinstance(a, sp.Expr):
-                if isinstance(a, Number):
-                    sc *= float(a)
-                else:
-                    ms *= a
-
-            elif isinstance(a, Number):
-                sc *= a
+                    sc[i] = sc[i] * sp.sympify(a[i])
 
             else:
-                raise NotImplementedError
+                sc *= sp.sympify(a)
 
-        return Expr(self._basis, self._terms.copy(), sc, self._indices.copy(), ms)
+        return Expr(self._basis, self._terms.copy(), sc, self._indices.copy())
 
     def __rmul__(self, a):
         return self.__mul__(a)
 
     def __imul__(self, a):
         sc = self.scales()
-        ms = self.measures()
         if self.expr_rank() == 0:
-            if isinstance(a, sp.Expr):
-                if isinstance(a, Number):
-                    sc *= float(a)
-                else:
-                    ms *= a
-            elif isinstance(a, Number):
-                sc *= a
+            sc *= sp.sympify(a)
 
         else:
             if isinstance(a, tuple):
                 assert len(a) == self.dimensions
                 for i in range(self.dimensions):
-                    if isinstance(a[i], sp.Expr):
-                        if isinstance(a[i], Number):
-                            sc[i] = sc[i]*float(a[i])
-                        else:
-                            ms[i] = ms[i]*a[i]
-                    elif isinstance(a[i], Number):
-                        sc[i] = sc[i]*a[i]
-
-                    else:
-                        raise NotImplementedError
-            elif isinstance(a, sp.Expr):
-                if isinstance(a, Number):
-                    sc *= float(a)
-                else:
-                    ms *= a
-
-            elif isinstance(a, Number):
-                sc *= a
+                    sc[i] = sc[i] * sp.sympify(a[i])
 
             else:
-                raise NotImplementedError
+                sc *= sp.sympify(a)
 
         return self
 
@@ -580,8 +559,7 @@ class Expr(object):
         return Expr(basis,
                     np.concatenate((self.terms(), a.terms()), axis=1),
                     np.concatenate((self.scales(), a.scales()), axis=1),
-                    np.concatenate((self.indices(), a.indices()), axis=1),
-                    np.concatenate((self.measures(), a.measures()), axis=1))
+                    np.concatenate((self.indices(), a.indices()), axis=1))
 
     def __iadd__(self, a):
         assert isinstance(a, (Expr, BasisFunction))
@@ -599,7 +577,6 @@ class Expr(object):
         self._terms = np.concatenate((self.terms(), a.terms()), axis=1)
         self._scales = np.concatenate((self.scales(), a.scales()), axis=1)
         self._indices = np.concatenate((self.indices(), a.indices()), axis=1)
-        self._measures = np.concatenate((self.measures(), a.measures()), axis=1)
         return self
 
     def __sub__(self, a):
@@ -617,8 +594,7 @@ class Expr(object):
         return Expr(basis,
                     np.concatenate((self.terms(), a.terms()), axis=1),
                     np.concatenate((self.scales(), -a.scales()), axis=1),
-                    np.concatenate((self.indices(), a.indices()), axis=1),
-                    np.concatenate((self.measures(), a.measures()), axis=1))
+                    np.concatenate((self.indices(), a.indices()), axis=1))
 
     def __isub__(self, a):
         assert isinstance(a, (Expr, BasisFunction))
@@ -636,12 +612,11 @@ class Expr(object):
         self._terms = np.concatenate((self.terms(), a.terms()), axis=1)
         self._scales = np.concatenate((self.scales(), -a.scales()), axis=1)
         self._indices = np.concatenate((self.indices(), a.indices()), axis=1)
-        self._measures = np.concatenate((self.measures(), a.measures()), axis=1)
         return self
 
     def __neg__(self):
         return Expr(self.basis(), self.terms().copy(), -self.scales().copy(),
-                    self.indices().copy(), self.measures().copy())
+                    self.indices().copy())
 
 
 class BasisFunction(object):
@@ -873,7 +848,8 @@ class ShenfunBaseArray(DistArray):
             for sym in sym0:
                 j = 'xyzrs'.index(str(sym))
                 m.append(mesh[j])
-            buffer = sp.lambdify(sym0, buffer)(*m)
+
+            buffer = sp.lambdify(sym0, buffer, modules=['numpy', {'cot': cot, 'Ynm': Ynm}])(*m)
             if cls.__name__ == 'Function':
                 buf = np.empty_like(space.forward.output_array)
                 buf = space.forward(buffer, buf)
