@@ -42,12 +42,21 @@ class LegendreBase(SpectralBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="LG", domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         SpectralBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.forward = functools.partial(self.forward, fast_transform=False)
         self.backward = functools.partial(self.backward, fast_transform=False)
         self.scalar_product = functools.partial(self.scalar_product, fast_transform=False)
@@ -77,7 +86,7 @@ class LegendreBase(SpectralBase):
     def vandermonde(self, x):
         return leg.legvander(x, self.N-1)
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         return sympy.legendre(i, x)
 
     def evaluate_basis(self, x, i=0, output_array=None):
@@ -149,7 +158,7 @@ class LegendreBase(SpectralBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_orthogonal(self):
-        return Basis(self.N, quad=self.quad, domain=self.domain)
+        return Basis(self.N, quad=self.quad, domain=self.domain, coordinates=self.coors.coordinates)
 
 @inheritdocstrings
 class Basis(LegendreBase):
@@ -170,12 +179,21 @@ class Basis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="LG", domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.plan(int(padding_factor*N), 0, np.float, {})
 
     def eval(self, x, u, output_array=None):
@@ -215,11 +233,20 @@ class ShenDirichletBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="LG", bc=(0., 0.), domain=(-1., 1.), scaled=False,
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         from shenfun.tensorproductspace import BoundaryValues
         self.LT = Basis(N, quad)
         self._scaled = scaled
@@ -277,7 +304,7 @@ class ShenDirichletBasis(LegendreBase):
     def slice(self):
         return slice(0, self.N-2)
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         f = sympy.legendre(i, x)-sympy.legendre(i+2, x)
         if self.is_scaled():
             f /= np.sqrt(4*i+6)
@@ -365,7 +392,8 @@ class ShenDirichletBasis(LegendreBase):
         self.sl = slicedict(axis=self.axis, dimensions=self.dimensions)
 
     def get_bc_basis(self):
-        return BCBasis(self.N, quad=self.quad, domain=self.domain, scaled=self._scaled)
+        return BCBasis(self.N, quad=self.quad, domain=self.domain,
+                       scaled=self._scaled, coordinates=self.coors.coordinates)
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         return ShenDirichletBasis(self.N,
@@ -373,6 +401,7 @@ class ShenDirichletBasis(LegendreBase):
                                   padding_factor=padding_factor,
                                   dealias_direct=dealias_direct,
                                   domain=self.domain,
+                                  coordinates=self.coors.coordinates,
                                   bc=self.bc.bc)
 
 @inheritdocstrings
@@ -397,12 +426,21 @@ class ShenNeumannBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="LG", mean=0, domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.mean = mean
         self.LT = Basis(N, quad)
         self._factor = np.zeros(0)
@@ -430,7 +468,7 @@ class ShenNeumannBasis(LegendreBase):
         output[self.sl[slice(-2, None)]] = 0
         return output
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         f = sympy.legendre(i, x)-(i*(i+1))/((i+2)*(i+3))*sympy.legendre(i+2, x)
         return f
 
@@ -539,12 +577,21 @@ class ShenBiharmonicBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="LG", bc=(0, 0, 0, 0), domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         from shenfun.tensorproductspace import BoundaryValues
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.LT = Basis(N, quad)
         self._factor1 = np.zeros(0)
         self._factor2 = np.zeros(0)
@@ -589,7 +636,7 @@ class ShenBiharmonicBasis(LegendreBase):
         w_hat[s4] += f2*fk[s]
         return w_hat
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if i < self.N-4:
             f = (sympy.legendre(i, x)
                  -2*(2*i+5.)/(2*i+7.)*sympy.legendre(i+2, x)
@@ -607,7 +654,7 @@ class ShenBiharmonicBasis(LegendreBase):
         if i < self.N-4:
             output_array[:] = eval_legendre(i, x) - 2*(2*i+5.)/(2*i+7.)*eval_legendre(i+2, x) + ((2*i+3.)/(2*i+7.))*eval_legendre(i+4, x)
         else:
-            X = sympy.symbols('x')
+            X = sympy.symbols('x', real=True)
             output_array[:] = sympy.lambdify(X, self.sympy_basis(i, x=X))(x)
         return output_array
 
@@ -625,7 +672,7 @@ class ShenBiharmonicBasis(LegendreBase):
                 basis = basis.deriv(k)
             output_array[:] = basis(x)
         else:
-            X = sympy.symbols('x')
+            X = sympy.symbols('x', real=True)
             output_array[:] = sympy.lambdify(X, self.sympy_basis(i, X).diff(X, k))(x)
         return output_array
 
@@ -667,7 +714,8 @@ class ShenBiharmonicBasis(LegendreBase):
         return self.forward.output_array
 
     def get_bc_basis(self):
-        return BCBiharmonicBasis(self.N, quad=self.quad, domain=self.domain)
+        return BCBiharmonicBasis(self.N, quad=self.quad, domain=self.domain,
+                                 coordinates=self.coors.coordinates)
 
     def plan(self, shape, axis, dtype, options):
         if isinstance(axis, tuple):
@@ -715,12 +763,21 @@ class UpperDirichletBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="LG", domain=(-1., 1.),
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         assert quad == "LG"
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.LT = Basis(N, quad)
         self._factor = np.ones(1)
         self.plan(int(N*padding_factor), 0, np.float, {})
@@ -753,7 +810,7 @@ class UpperDirichletBasis(LegendreBase):
     def slice(self):
         return slice(0, self.N-1)
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         f = sympy.legendre(i, x)-sympy.legendre(i+1, x)
         return f
 
@@ -833,6 +890,7 @@ class UpperDirichletBasis(LegendreBase):
                                   quad=self.quad,
                                   padding_factor=padding_factor,
                                   dealias_direct=dealias_direct,
+                                  coordinates=self.coors.coordinates,
                                   domain=self.domain)
 
 @inheritdocstrings
@@ -855,12 +913,21 @@ class ShenBiPolarBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="LG", domain=(-1., 1.),
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         assert quad == "LG"
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.LT = Basis(N, quad)
         self.plan(int(N*padding_factor), 0, np.float, {})
 
@@ -893,11 +960,9 @@ class ShenBiPolarBasis(LegendreBase):
     def evaluate_basis_all(self, x=None, argument=0):
         if x is None:
             x = self.mesh(False, False)
-        N, M = self.shape(False), self.shape(True)
-        output_array = np.zeros((M, N))
-        D = np.zeros(M)
-        for j in range(N-4):
-            output_array[:, j] = self.evaluate_basis(x, j, D)
+        output_array = np.zeros((x.shape[0], self.N))
+        for j in range(self.N-4):
+            output_array[:, j] = self.evaluate_basis(x, j, output_array=output_array[:, j])
         return output_array
 
     def evaluate_basis_derivative(self, x=None, i=0, k=0, output_array=None):
@@ -906,20 +971,18 @@ class ShenBiPolarBasis(LegendreBase):
         if output_array is None:
             output_array = np.zeros(x.shape)
         x = np.atleast_1d(x)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, X).diff(X, k)
         output_array[:] = sympy.lambdify(X, f)(x)
         return output_array
 
     def evaluate_basis_derivative_all(self, x=None, k=0, argument=0):
         if x is None:
-            x = self.mesh(False, False)
-        N, M = self.shape(False), self.shape(True)
-        output_array = np.zeros((M, N))
-        D = np.zeros(M)
-        for j in range(N-4):
-            output_array[:, j] = self.evaluate_basis_derivative(x, j, k, D)
-        return output_array
+            x = self.mpmath_points_and_weights()[0]
+        V = np.zeros((x.shape[0], self.N))
+        for i in range(self.N-2):
+            V[:, i] = self.evaluate_basis_derivative(x, i, k, output_array=V[:, i])
+        return V
 
     def vandermonde_scalar_product(self, input_array, output_array):
         SpectralBase.vandermonde_scalar_product(self, input_array, output_array)
@@ -928,9 +991,11 @@ class ShenBiPolarBasis(LegendreBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
+        else:
+            output_array.fill(0)
         x = self.map_reference_domain(x)
-        fj = self.eval_basis_all(x)
-        output_array[:] = np.dot(u, fj)
+        fj = self.evaluate_basis_all(x)
+        output_array[:] = np.dot(fj, u)
         return output_array
 
     def forward(self, input_array=None, output_array=None, fast_transform=False):
@@ -974,6 +1039,7 @@ class ShenBiPolarBasis(LegendreBase):
                                 quad=self.quad,
                                 padding_factor=padding_factor,
                                 dealias_direct=dealias_direct,
+                                coordinates=self.coors.coordinates,
                                 domain=self.domain)
 
 @inheritdocstrings
@@ -999,12 +1065,21 @@ class ShenBiPolar0Basis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
     def __init__(self, N, quad="LG", domain=(-1., 1.), padding_factor=1,
-                 dealias_direct=False):
+                 dealias_direct=False, coordinates=None):
         assert quad == "LG"
         LegendreBase.__init__(self, N, quad="LG", domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.LT = Basis(N, quad)
         self._factor1 = np.zeros(0)
         self._factor2 = np.zeros(0)
@@ -1051,7 +1126,6 @@ class ShenBiPolar0Basis(LegendreBase):
         return w_hat
 
     def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
-        assert i < self.N-3
         x = self.map_reference_domain(x)
         return (sympy.legendre(i, x)
                 -(2*i+3)*(i+4)/(2*i+5)/(i+2)*sympy.legendre(i+1, x)
@@ -1231,8 +1305,8 @@ class ShenBiPolar0Basis(LegendreBase):
 class BCBasis(LegendreBase):
 
     def __init__(self, N, quad="LG", scaled=False,
-                 domain=(-1., 1.)):
-        LegendreBase.__init__(self, N, quad=quad, domain=domain)
+                 domain=(-1., 1.), coordinates=None):
+        LegendreBase.__init__(self, N, quad=quad, domain=domain, coordinates=coordinates)
         self._scaled = scaled
         self.plan(N, 0, np.float, {})
 
@@ -1284,7 +1358,7 @@ class BCBasis(LegendreBase):
         P[:, 1] = (V[:, 0] + V[:, 1])/2
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if i == 0:
             return 0.5*(1-x)
         elif i == 1:
@@ -1337,12 +1411,21 @@ class BCBiharmonicBasis(LegendreBase):
             Factor for padding backward transforms.
         dealias_direct : bool, optional
             Set upper 1/3 of coefficients to zero before backward transform
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
     """
 
     def __init__(self, N, quad="GC", domain=(-1., 1.),
-                 padding_factor=1, dealias_direct=False):
+                 padding_factor=1, dealias_direct=False, coordinates=None):
         LegendreBase.__init__(self, N, quad=quad, domain=domain,
-                              padding_factor=padding_factor, dealias_direct=dealias_direct)
+                              padding_factor=padding_factor, dealias_direct=dealias_direct,
+                              coordinates=coordinates)
         self.plan(N, 0, np.float, {})
 
     def plan(self, shape, axis, dtype, options):
@@ -1392,7 +1475,7 @@ class BCBiharmonicBasis(LegendreBase):
         P = np.tensordot(V[:, :4], self.coefficient_matrix(), (1, 1))
         return P
 
-    def sympy_basis(self, i=0, x=sympy.symbols('x')):
+    def sympy_basis(self, i=0, x=sympy.symbols('x', real=True)):
         if i < 4:
             f = 0
             for j, c in enumerate(self.coefficient_matrix()[i]):
@@ -1405,7 +1488,7 @@ class BCBiharmonicBasis(LegendreBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, X)
         output_array[:] = sympy.lambdify(X, f)(x)
         return output_array
@@ -1414,7 +1497,7 @@ class BCBiharmonicBasis(LegendreBase):
         x = np.atleast_1d(x)
         if output_array is None:
             output_array = np.zeros(x.shape)
-        X = sympy.symbols('x')
+        X = sympy.symbols('x', real=True)
         f = self.sympy_basis(i, X)
         output_array[:] = sympy.lambdify(X, f.diff(X, k))(x)
         return output_array
