@@ -208,7 +208,8 @@ import warnings
 import sympy as sp
 import numpy as np
 from mpi4py_fft import fftw
-from .utilities import CachedArrayDict
+from .utilities import CachedArrayDict, split
+from .coordinates import Coordinates
 work = CachedArrayDict()
 
 class SpectralBase(object):
@@ -233,11 +234,22 @@ class SpectralBase(object):
         dealias_direct : bool, optional
             If True, then set all upper 2/3 wavenumbers to zero before backward
             transform. Cannot be used if padding_factor is different than 1.
+        cordinates: 2-tuple (coordinate, position vector), optional
+        Map for curvilinear coordinatesystem.
+        The new coordinate variable in the new coordinate system is the first item.
+        Second item is a tuple for the Cartesian position vector as function of the
+        new variable in the first tuple. Example::
+
+            theta = sp.Symbols('x', real=True, positive=True)
+            rv = (sp.cos(theta), sp.sin(theta))
+
+        where theta and rv are the first and second items in the 2-tuple.
 
     """
     # pylint: disable=method-hidden, too-many-instance-attributes
 
-    def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.), dealias_direct=False):
+    def __init__(self, N, quad='', padding_factor=1, domain=(-1., 1.),
+                 dealias_direct=False, coordinates=None):
         self.N = N
         self.domain = domain
         self.quad = quad
@@ -251,7 +263,10 @@ class SpectralBase(object):
         self._xfftn_fwd = None    # external forward transform function
         self._xfftn_bck = None    # external backward transform function
         self._M = 1.0             # Normalization factor
-        self._dx = 1              # Integral measure (in addition to weight)
+        self.hi = np.ones(1, dtype=object)  # Integral measure (in addition to weight)
+        coors = coordinates if coordinates is not None else ((sp.Symbol('x', real=True),),)*2
+        self.coors = Coordinates(*coors)
+        self.hi = self.coors.get_scaling_factors()
         self.si = islicedict()
         self.sl = slicedict()
         self._tensorproductspace = None     # link if belonging to TensorProductSpace
@@ -321,10 +336,27 @@ class SpectralBase(object):
         else:
             d = self.domain
             X = np.linspace(d[0], d[1], N)
+            if map_true_domain is False:
+                X = self.map_reference_domain(X)
         if bcast is True:
             X = self.broadcast_to_ndims(X)
         return X
 
+    def curvilinear_mesh(self, uniform=False):
+        """Return curvilinear mesh of of basis
+
+        Parameters
+        ----------
+        uniform : bool, optional
+            Use uniform mesh
+        """
+        x = self.mesh(uniform=uniform)
+        psi = self.coors.coordinates[0]
+        xx = []
+        for rv in self.coors.coordinates[1]:
+            xx.append(sp.lambdify(psi, rv)(x))
+        return xx
+
     def wavenumbers(self, bcast=True, **kw):
         """Return the wavenumbermesh
 
@@ -495,7 +527,7 @@ class SpectralBase(object):
 
         self.vandermonde_evaluate_expansion_all(self.backward.tmp_array,
                                                 self.backward.output_array,
-                                                x=self.mesh(bcast=False, uniform=True))
+                                                x=self.mesh(bcast=False, map_true_domain=False, uniform=True))
 
         if output_array is not None:
             output_array[...] = self.backward.output_array
@@ -1034,14 +1066,23 @@ class SpectralBase(object):
 
     def get_mass_matrix(self):
         mat = self._get_mat()
-        dx = self._dx
+        dx = self.hi.prod()
+        key = ((self.__class__, 0), (self.__class__, 0))
+        if self.tensorproductspace:
+            dx = self.tensorproductspace.hi.prod()
+            msdict = split(dx)
+            assert len(msdict) == 1
+            dx = msdict[0]['xyzrs'[self.axis]]
         if not dx == 1:
             x0 = dx.free_symbols
-            assert len(x0) == 1
+            if len(x0) > 1:
+                raise NotImplementedError("Cannot use forward for Curvilinear coordinates with unseparable measure - Use inner with mass matrix for tensor product space")
             x0 = x0.pop()
             x = sp.symbols('x', real=x0.is_real, positive=x0.is_positive)
             dx = dx.subs(x0, x)
-        return mat[((self.__class__, 0), (self.__class__, 0), dx)]
+            key = key + (self.domain, dx)
+
+        return mat[key]
 
     def _get_mat(self):
         mod = importlib.import_module('shenfun.'+self.family())
@@ -1060,7 +1101,7 @@ class SpectralBase(object):
     def __hash__(self):
         return hash((self.N, self.quad, self.family()))
 
-    def get_measured_weights(self, N=None, measure=None):
+    def get_measured_weights(self, N=None, measure=1):
         """Return weights times `measure`
 
         Parameters
@@ -1071,38 +1112,53 @@ class SpectralBase(object):
         """
         if N is None:
             N = self.N
-        dx = self._dx if measure is None else measure
         xm, wj = self.mpmath_points_and_weights(N, map_true_domain=True)
-        if dx == 1:
+        if measure == 1:
             return wj
 
-        s = dx.free_symbols
+        s = measure.free_symbols
         assert len(s) == 1
         s = s.pop()
-        xj = sp.lambdify(s, dx)(xm)
+        xj = sp.lambdify(s, measure)(xm)
+        if wj.shape[0] == 1:
+            wj = np.broadcast_to(wj, xj.shape).copy()
         wj *= xj
         return wj
 
-    def get_measured_array(self, input_array, measure=None):
-        """Return `input_array` times `measure`
+    def get_measured_array(self, array):
+        """Return weights times `measure`
 
         Parameters
         ----------
-        input_array : Array
+        N : integer, optional
+            The number of quadrature points
         measure : None or `sympy.Expr`
-        """
-        dx = self._dx if measure is None else measure
-        xm = self.mpmath_points_and_weights(self.N, map_true_domain=True)[0]
-        if dx == 1:
-            return input_array
 
-        s = dx.free_symbols
-        assert len(s) == 1
-        s = s.pop()
-        xj = sp.lambdify(s, dx)(xm)
-        xj = self.broadcast_to_ndims(xj)
-        input_array *= xj
-        return input_array
+        Note
+        ----
+        If basis is part of a `TensorProductSpace`, then the
+        array will be measured there. So in that case, just return
+        the array unchanged.
+        """
+        if self.tensorproductspace:
+            return array
+
+        measure = self.hi.prod()
+        if measure == 1:
+            return array
+
+        xm = self.mpmath_points_and_weights(self.N, map_true_domain=True)[0]
+        s = measure.free_symbols
+        if len(s) == 0:
+            # constant
+            array[...] = array*float(measure)
+        elif len(s) == 1:
+            s = s.pop()
+            xj = sp.lambdify(s, measure)(xm)
+            array[...] = array*xj
+        else:
+            raise NotImplementedError
+        return array
 
     def get_dealiased(self, padding_factor=1.5, dealias_direct=False):
         """Return space (otherwise as self) to be used for dealiasing
@@ -1124,7 +1180,8 @@ class SpectralBase(object):
                               quad=self.quad,
                               domain=self.domain,
                               padding_factor=padding_factor,
-                              dealias_direct=dealias_direct)
+                              dealias_direct=dealias_direct,
+                              coordinates=self.coors.coordinates)
 
     def get_refined(self, N):
         """Return space (otherwise as self) with N quadrature points
@@ -1143,7 +1200,8 @@ class SpectralBase(object):
                               quad=self.quad,
                               domain=self.domain,
                               padding_factor=self.padding_factor,
-                              dealias_direct=self.dealias_direct)
+                              dealias_direct=self.dealias_direct,
+                              coordinates=self.coors.coordinates)
 
     def _truncation_forward(self, padded_array, trunc_array):
         if not id(trunc_array) == id(padded_array):
