@@ -4,13 +4,15 @@ related methods.
 """
 from numbers import Number
 import warnings
+import functools
 import sympy as sp
 import numpy as np
 from shenfun.fourier.bases import R2CBasis, C2CBasis
-from shenfun.utilities import apply_mask, get_measures, split
+from shenfun.utilities import apply_mask, split
 from shenfun.forms.arguments import Function, Array
 from shenfun.optimization.cython import evaluate
 from shenfun.spectralbase import slicedict, islicedict, SpectralBase
+from shenfun.coordinates import Coordinates
 from mpi4py_fft.mpifft import Transform, PFFT
 from mpi4py_fft.pencil import Subcomm, Pencil
 
@@ -20,6 +22,91 @@ __all__ = ('TensorProductSpace', 'VectorTensorProductSpace',
 # Default sympy symbols. Note that order is important
 x, y, z, r, s = psi = sp.symbols('x,y,z,r,s', real=True)
 
+class CurvilinearTransform(Transform):
+    """Class for performing forward parallel transform using curvilinear
+    coordinates
+
+    Parameters
+    ----------
+    xfftn : list of serial transform objects
+    transfer : list of global redistribution objects
+    pencil : list of two pencil objects
+        The two pencils represent the input and final output configuration of
+        the distributed global arrays
+    T : TensorProductSpace
+    """
+    def __init__(self, xfftn, transfer, pencil, T=None):
+        assert len(xfftn) == len(transfer) + 1 and len(pencil) == 2
+        assert T is not None
+        self._xfftn = tuple(xfftn)
+        self._transfer = tuple(transfer)
+        self._pencil = tuple(pencil)
+        self._T = T
+
+    @property
+    def hi(self):
+        """Return scaling factors of Transform"""
+        return self._T.hi
+
+    @property
+    def local_mesh(self):
+        """Return local mesh"""
+        return self._T.local_mesh(True)
+
+    def get_measured_input_array(self):
+        """Weigh input array with integral measure
+        """
+        dx = self.hi.prod()
+        mesh = self.local_mesh
+        if dx == 1:
+            return
+
+        sym0 = dx.free_symbols
+        m = []
+        for sym in sym0:
+            j = 'xyzrs'.index(str(sym))
+            m.append(mesh[j])
+        xj = sp.lambdify(sym0, dx)(*m)
+        self.input_array[...] = self.input_array*xj
+        return
+
+    def __call__(self, input_array=None, output_array=None, **kw):
+        """Compute transform
+
+        Parameters
+        ----------
+        input_array : array, optional
+        output_array : array, optional
+        kw : dict
+            parameters to serial transforms
+            Note in particular that the keyword 'normalize'=True/False can be
+            used to turn normalization on or off. Default is to enable
+            normalization for forward transforms and disable it for backward.
+
+        Note
+        ----
+        If input_array/output_array are not given, then use predefined arrays
+        as planned with serial transform object _xfftn.
+
+        """
+        if input_array is not None:
+            self.input_array[...] = input_array
+
+        self.get_measured_input_array()
+
+        for i in range(len(self._transfer)):
+            self._xfftn[i](**kw)
+            arrayA = self._xfftn[i].output_array
+            arrayB = self._xfftn[i+1].input_array
+            self._transfer[i](arrayA, arrayB)
+        self._xfftn[-1](**kw)
+
+        if output_array is not None:
+            output_array[...] = self.output_array
+            return output_array
+        else:
+            return self.output_array
+
 class TensorProductSpace(PFFT):
     """Class for multidimensional tensorproductspaces.
 
@@ -47,6 +134,17 @@ class TensorProductSpace(PFFT):
         Pencil distribution in spectral space. This is primarily intended
         for a padded space, where the spectral distribution must be
         equal to the non-padded space.
+    cordinates: two tuples, optional
+        Map for curvilinear coordinatesystem.
+        First tuple are the coordinate variables in the new coordinate system
+        Second tuple are the Cartesian coordinates as functions of the variables
+        in the first tuple. Example::
+
+            psi = (theta, r) = sp.symbols('x,y', real=True, positive=True)
+            rv = (r*sp.cos(theta), r*sp.sin(theta))
+
+        where psi and rv are the first and second tuples, respectively.
+
     kw : dict, optional
         Dictionary that can be used to plan transforms. Input to method
         `plan` for the bases.
@@ -54,17 +152,13 @@ class TensorProductSpace(PFFT):
     """
     def __init__(self, comm, bases, axes=None, dtype=None, slab=False,
                  collapse_fourier=False, backward_from_pencil=False,
-                 measures=None, **kw):
+                 coordinates=None, **kw):
         # Note do not call __init__ of super
         self.comm = comm
         self.bases = bases
-        self.measures = measures if measures is not None else (psi[:len(bases)],)*2
-        self.hi = get_measures(*self.measures)
-        if not self.hi.prod() == 1:
-            for key, val in split(self.hi).items():
-                k = 'xyzrs'.index(key)
-                self.bases[k]._dx *= val
-
+        coors = coordinates if coordinates is not None else (psi[:len(bases)],)*2
+        self.coors = Coordinates(*coors)
+        self.hi = self.coors.get_scaling_factors()
         shape = list(self.global_shape())
         assert shape
         assert min(shape) > 0
@@ -178,7 +272,9 @@ class TensorProductSpace(PFFT):
 
             self.pencil[1] = pencilA
 
-            self.forward = Transform(
+            FT = functools.partial(CurvilinearTransform, T=self) if not self.hi.prod() == 1 else Transform
+
+            self.forward = FT(
                 [o.forward for o in self.xfftn],
                 [o.forward for o in self.transfer],
                 self.pencil)
@@ -186,7 +282,7 @@ class TensorProductSpace(PFFT):
                 [o.backward for o in self.xfftn[::-1]],
                 [o.backward for o in self.transfer[::-1]],
                 self.pencil[::-1])
-            self.scalar_product = Transform(
+            self.scalar_product = FT(
                 [o.scalar_product for o in self.xfftn],
                 [o.forward for o in self.transfer],
                 self.pencil)
@@ -262,15 +358,17 @@ class TensorProductSpace(PFFT):
 
         self.pencil[1] = pencilA
 
+        FT = functools.partial(CurvilinearTransform, T=self) if not self.hi.prod() == 1 else Transform
+
         self.backward = Transform(
             [o.backward for o in self.xfftn],
             [o.forward for o in self.transfer],
             self.pencil)
-        self.forward = Transform(
+        self.forward = FT(
             [o.forward for o in self.xfftn[::-1]],
             [o.backward for o in self.transfer[::-1]],
             self.pencil[::-1])
-        self.scalar_product = Transform(
+        self.scalar_product = FT(
             [o.scalar_product for o in self.xfftn[::-1]],
             [o.forward for o in self.transfer[::-1]],
             self.pencil[::-1])
@@ -289,7 +387,8 @@ class TensorProductSpace(PFFT):
                         for axis, base in enumerate(self.bases)]
         return TensorProductSpace(self.comm, padded_bases,
                                   dtype=self.forward.output_array.dtype,
-                                  backward_from_pencil=self.forward.output_pencil)
+                                  backward_from_pencil=self.forward.output_pencil,
+                                  coordinates=self.coors.coordinates)
 
     def get_refined(self, N):
         if isinstance(N, Number):
@@ -298,7 +397,8 @@ class TensorProductSpace(PFFT):
             assert len(N) == len(self)
         refined_bases = [base.get_refined(N[axis])
                          for axis, base in enumerate(self.bases)]
-        return TensorProductSpace(self.comm, refined_bases, axes=self.axes)
+        return TensorProductSpace(self.comm, refined_bases, axes=self.axes,
+                                  coordinates=self.coors.coordinates)
 
     def convolve(self, a_hat, b_hat, ab_hat):
         """Convolution of a_hat and b_hat
@@ -576,7 +676,7 @@ class TensorProductSpace(PFFT):
             X.append(base.mesh(uniform=uniform))
         return X
 
-    def local_mesh(self, broadcast=False):
+    def local_mesh(self, broadcast=False, uniform=False):
         """Return list of local 1D physical meshes for each dimension of
         TensorProductSpace
 
@@ -585,8 +685,10 @@ class TensorProductSpace(PFFT):
         broadcast : bool, optional
             Broadcast each 1D mesh to real shape of
             :class:`.TensorProductSpace`
+        uniform : bool, optional
+            Use uniform mesh for non-periodic bases
         """
-        mesh = self.mesh()
+        mesh = self.mesh(uniform=uniform)
         lm = []
         for axis, (n, s) in enumerate(zip(mesh, self.local_slice(False))):
             ss = [slice(None)]*len(mesh)
@@ -596,6 +698,36 @@ class TensorProductSpace(PFFT):
             return [np.broadcast_to(m, self.shape(False)) for m in lm]
         return lm
 
+    def local_curvilinear_mesh(self, uniform=False):
+        """Return curvilinear mesh
+
+        Parameters
+        ----------
+        uniform : bool, optional
+            Use uniform mesh for non-periodic bases
+        """
+        X = self.local_mesh(broadcast=True, uniform=uniform)
+        xx = []
+        psi = self.coors.coordinates[0]
+        for rv in self.coors.coordinates[1]:
+            xx.append(sp.lambdify(psi, rv)(*X))
+        return xx
+
+    def curvilinear_mesh(self, uniform=False):
+        """Return curvilinear mesh
+
+        Parameters
+        ----------
+        uniform : bool, optional
+            Use uniform mesh for non-periodic bases
+        """
+        X = self.mesh(uniform=uniform)
+        xx = []
+        psi = self.coors.coordinates[0]
+        for rv in self.coors.coordinates[1]:
+            xx.append(sp.lambdify(psi, rv)(*X))
+        return xx
+
     def dim(self):
         """Return dimension of ``self`` (degrees of freedom)"""
         return np.prod(self.dims())
@@ -625,12 +757,12 @@ class TensorProductSpace(PFFT):
         #return self.shape(forward_output)
 
     def mask_nyquist(self, u_hat, mask=None):
-        """Return array `u_hat` with zero Nyquist coefficients
+        """Return Function `u_hat` with zero Nyquist coefficients
 
         Parameters
         ----------
         u_hat : array
-            Array to be masked
+            Function to be masked
         mask : array or None, optional
             mask array, if not provided then get the mask by calling
             :func:`get_mask_nyquist`
@@ -669,6 +801,27 @@ class TensorProductSpace(PFFT):
             mask = mask * ll
         return mask
 
+    def get_measured_array(self, u):
+        """Weigh Array `u` with integral measure
+
+        Parameters
+        ----------
+        u : Array
+        """
+        dx = self.hi.prod()
+        mesh = self.local_mesh(True)
+        if dx == 1:
+            return
+
+        sym0 = dx.free_symbols
+        m = []
+        for sym in sym0:
+            j = 'xyzrs'.index(str(sym))
+            m.append(mesh[j])
+        xj = sp.lambdify(sym0, dx)(*m)
+        u *= xj
+        return u
+
     def __iter__(self):
         return iter(self.bases)
 
