@@ -13,8 +13,7 @@ from scipy.fftpack import dct
 from shenfun.optimization import optimizer
 
 __all__ = ['inheritdocstrings', 'dx', 'clenshaw_curtis1D', 'CachedArrayDict',
-           'outer', 'apply_mask', 'integrate_sympy', 'get_measures',
-           'get_measures_tangent']
+           'outer', 'apply_mask', 'integrate_sympy']
 
 def inheritdocstrings(cls):
     """Method used for inheriting docstrings from parent class
@@ -219,27 +218,13 @@ def integrate_sympy(f, d):
     except sp.PolynomialError:
         return sp.integrate(f, d)
 
-def get_measures(psi, rv):
-    drv = get_measures_tangent(psi, rv)
-    measures = np.zeros_like(rv)
-    for i, s in enumerate(np.sum(drv**2, axis=1)):
-        measures[i] = sp.simplify(sp.sqrt(s))
-    return measures
-
-def get_measures_tangent(psi, rv):
-    drv = np.zeros((len(rv),)*2, dtype=object)
-    for i, ti in enumerate(psi):
-        for j, rj in enumerate(rv):
-            drv[i, j] = rj.diff(ti, 1)
-    return drv
-
 def split(measures):
     def _split(mss, result):
         for ms in mss:
             ms = sp.sympify(ms)
             if isinstance(ms, sp.Mul):
                 # Multiplication of two or more terms
-                _split(ms.args, result)
+                result = _split(ms.args, result)
                 continue
 
             # Something else with only one symbol
@@ -249,7 +234,15 @@ def split(measures):
                 sym = sym.pop()
                 result[str(sym)] *= ms
             else:
-                result['x'] *= ms
-    result = defaultdict(lambda: 1)
-    _split(measures, result)
+                ms = int(ms) if isinstance(ms, sp.Integer) else float(ms)
+                result['scale'] *= ms
+        return result
+
+    ms = sp.sympify(measures).expand()
+    result = []
+    if isinstance(ms, sp.Add):
+        for arg in ms.args:
+            result.append(_split([arg], defaultdict(lambda: 1)))
+    else:
+        result.append(_split([ms], defaultdict(lambda: 1)))
     return result
